package contact

import (
	"net/http"

	"github.com/zeromicro/go-zero/rest/httpx"

	"wechat-api/internal/logic/contact"
	"wechat-api/internal/svc"
	"wechat-api/internal/types"
)

// swagger:route post /contact/list contact GetContactList
//
// Get contact list | 获取Contact列表
//
// Get contact list | 获取Contact列表
//
// Parameters:
//  + name: body
//    require: true
//    in: body
//    type: ContactListReq
//
// Responses:
//  200: ContactListResp

func GetContactListHandler(svcCtx *svc.ServiceContext) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		var req types.ContactListReq
		if err := httpx.Parse(r, &req, true); err != nil {
			httpx.ErrorCtx(r.Context(), w, err)
			return
		}

		l := contact.NewGetContactListLogic(r.Context(), svcCtx)
		resp, err := l.GetContactList(&req)
		if err != nil {
			httpx.ErrorCtx(r.Context(), w, err)
		} else {
			httpx.OkJsonCtx(r.Context(), w, resp)
		}
	}
}
