package openapi

import (
	"net/http"

	"github.com/zeromicro/go-zero/rest/httpx"

	"wechat-api/internal/logic/openapi"
	"wechat-api/internal/svc"
	"wechat-api/internal/types"
)

// swagger:route post /openapi/contact/getContactByWxid openapi OpenapiGetContactByWxid
//
// 通过wxid获取联系人信息
//
// 通过wxid获取联系人信息
//
// Parameters:
//  + name: body
//    require: true
//    in: body
//    type: WxidReq
//
// Responses:
//  200: ContactInfoResp

func OpenapiGetContactByWxidHandler(svcCtx *svc.ServiceContext) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		var req types.WxidReq
		if err := httpx.Parse(r, &req, true); err != nil {
			httpx.ErrorCtx(r.Context(), w, err)
			return
		}

		l := openapi.NewOpenapiGetContactByWxidLogic(r.Context(), svcCtx)
		resp, err := l.OpenapiGetContactByWxid(&req)
		if err != nil {
			httpx.ErrorCtx(r.Context(), w, err)
		} else {
			httpx.OkJsonCtx(r.Context(), w, resp)
		}
	}
}
