package Wxhook

import (
	"context"
	"github.com/suyuan32/simple-admin-common/enum/errorcode"
	"github.com/suyuan32/simple-admin-common/msg/errormsg"
	"wechat-api/ent/predicate"
	"wechat-api/ent/wx"
	"wechat-api/hook"

	"wechat-api/internal/svc"
	"wechat-api/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type SendPicMsgLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewSendPicMsgLogic(ctx context.Context, svcCtx *svc.ServiceContext) *SendPicMsgLogic {
	return &SendPicMsgLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx}
}

func (l *SendPicMsgLogic) SendPicMsg(req *types.SendPicMsgReq) (resp *types.BaseMsgResp, err error) {

	//根据wx实体的wxid查询
	var predicates []predicate.Wx
	if req.WxWxid != nil {
		predicates = append(predicates, wx.WxidContains(*req.WxWxid))
	}

	wxInfo, err := l.svcCtx.DB.Wx.Query().Where(predicates...).Only(l.ctx)

	//根据wx实体的主键ID查询
	//wxInfo, err := l.svcCtx.DB.Wx.Get(l.ctx, *req.AgentWxId)

	l.Infof("wxInfo = %v", wxInfo)

	if err != nil {
		l.Error("查询微信信息失败", err)
		return
	}

	serverInfo, err := l.svcCtx.DB.Server.Get(l.ctx, wxInfo.ServerID)
	if err != nil {
		l.Error("查询服务器信息失败", err)
		return
	}

	hookClient := hook.NewHook(serverInfo.PrivateIP, serverInfo.AdminPort, wxInfo.Port)

	err = hookClient.SendPicMsg(*req.Wxid, *req.Picpath, *req.Diyfilename)

	if err != nil {
		l.Error("发送微信图片失败", err)
		return
	}

	resp = &types.BaseMsgResp{
		Msg:  errormsg.Success,
		Code: errorcode.OK,
	}

	return
}
