package message_records

import (
	"context"

	"wechat-api/ent/messagerecords"
	"wechat-api/ent/predicate"
	"wechat-api/internal/svc"
	"wechat-api/internal/types"
	"wechat-api/internal/utils/dberrorhandler"

    "github.com/suyuan32/simple-admin-common/msg/errormsg"

	"github.com/suyuan32/simple-admin-common/utils/pointy"
	"github.com/zeromicro/go-zero/core/logx"
)

type GetMessageRecordsListLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetMessageRecordsListLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetMessageRecordsListLogic {
	return &GetMessageRecordsListLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *GetMessageRecordsListLogic) GetMessageRecordsList(req *types.MessageRecordsListReq) (*types.MessageRecordsListResp, error) {
	var predicates []predicate.MessageRecords
	if req.BotWxid != nil {
		predicates = append(predicates, messagerecords.BotWxidContains(*req.BotWxid))
	}
	if req.ContactWxid != nil {
		predicates = append(predicates, messagerecords.ContactWxidContains(*req.ContactWxid))
	}
	if req.Content != nil {
		predicates = append(predicates, messagerecords.ContentContains(*req.Content))
	}
	data, err := l.svcCtx.DB.MessageRecords.Query().Where(predicates...).Page(l.ctx, req.Page, req.PageSize)

	if err != nil {
		return nil, dberrorhandler.DefaultEntError(l.Logger, err, req)
	}

	resp := &types.MessageRecordsListResp{}
	resp.Msg = errormsg.Success
	resp.Data.Total = data.PageDetails.Total

	for _, v := range data.List {
		resp.Data.Data = append(resp.Data.Data,
		types.MessageRecordsInfo{
			BaseIDInfo:    types.BaseIDInfo{
				Id:          &v.ID,
				CreatedAt:    pointy.GetPointer(v.CreatedAt.UnixMilli()),
				UpdatedAt:    pointy.GetPointer(v.UpdatedAt.UnixMilli()),
            },
			Status:	&v.Status,
			BotWxid:	&v.BotWxid,
			ContactId:	&v.ContactID,
			ContactType:	&v.ContactType,
			ContactWxid:	&v.ContactWxid,
			ContentType:	&v.ContentType,
			Content:	&v.Content,
			ErrorDetail:	&v.ErrorDetail,
			SendTime:	pointy.GetUnixMilliPointer(v.SendTime.UnixMilli()),
			SourceType:	&v.SourceType,
			SourceId:	&v.SourceID,
			SubSourceId:	&v.SubSourceID,
		})
	}

	return resp, nil
}
