package sop_node

import (
	"context"
	"fmt"

	"wechat-api/ent/predicate"
	"wechat-api/ent/sopnode"
	"wechat-api/internal/svc"
	"wechat-api/internal/types"
	"wechat-api/internal/utils/dberrorhandler"

	"github.com/suyuan32/simple-admin-common/msg/errormsg"

	"github.com/suyuan32/simple-admin-common/utils/pointy"
	"github.com/zeromicro/go-zero/core/logx"
)

type GetSopNodeListLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetSopNodeListLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetSopNodeListLogic {
	return &GetSopNodeListLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *GetSopNodeListLogic) GetSopNodeList(req *types.SopNodeListReq) (*types.SopNodeListResp, error) {
	var predicates []predicate.SopNode
	if req.StageId != nil {
		predicates = append(predicates, sopnode.StageID(*req.StageId))
	} else {
		return nil, dberrorhandler.DefaultEntError(l.Logger, fmt.Errorf("StageId 不能为空"), req)
	}
	data, err := l.svcCtx.DB.SopNode.Query().Where(predicates...).All(l.ctx)

	if err != nil {
		return nil, dberrorhandler.DefaultEntError(l.Logger, err, req)
	}

	resp := &types.SopNodeListResp{}
	resp.Msg = errormsg.Success

	for _, v := range data {
		var actionMessage []types.Action
		if len(v.ActionMessage) > 0 {
			actionMessage = make([]types.Action, len(v.ActionMessage))
			for i, condition := range v.ActionMessage {
				actionMessage[i] = types.Action{
					Type:    condition.Type,
					Content: condition.Content,
				}
			}
		}

		resp.Data.Data = append(resp.Data.Data,
			types.SopNodeInfo{
				BaseIDInfo: types.BaseIDInfo{
					Id:        &v.ID,
					CreatedAt: pointy.GetPointer(v.CreatedAt.UnixMilli()),
					UpdatedAt: pointy.GetPointer(v.UpdatedAt.UnixMilli()),
				},
				Status:        &v.Status,
				StageId:       &v.StageID,
				ParentId:      &v.ParentID,
				Name:          &v.Name,
				ConditionType: &v.ConditionType,
				ConditionList: v.ConditionList,
				ActionMessage: actionMessage,
				ActionLabel:   v.ActionLabel,
			})
	}

	return resp, nil
}
