package sop_stage

import (
	"context"
	"wechat-api/ent/custom_types"
	"wechat-api/ent/sopstage"

	"wechat-api/internal/svc"
	"wechat-api/internal/types"
	"wechat-api/internal/utils/dberrorhandler"

	"github.com/suyuan32/simple-admin-common/msg/errormsg"

	"github.com/zeromicro/go-zero/core/logx"
)

type CreateSopStageLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewCreateSopStageLogic(ctx context.Context, svcCtx *svc.ServiceContext) *CreateSopStageLogic {
	return &CreateSopStageLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *CreateSopStageLogic) CreateSopStage(req *types.SopStageInfo) (*types.SopStageCreateResp, error) {
	// 根据 task_id 查询 task 是否存在
	task, err := l.svcCtx.DB.SopTask.Get(l.ctx, *req.TaskId)
	if err != nil {
		return nil, dberrorhandler.DefaultEntError(l.Logger, err, req)
	}
	// 判断 task 的状态是否为 1
	if task.Status != 1 {
		return nil, dberrorhandler.DefaultEntError(l.Logger, err, req)
	}

	// 查询某 task_id 下的 stage 的数量
	count, err := l.svcCtx.DB.SopStage.Query().Where(sopstage.TaskIDEQ(*req.TaskId)).Count(l.ctx)
	count += 1

	var conditionList []custom_types.Condition
	if len(req.ConditionList) > 0 {
		conditionList = make([]custom_types.Condition, len(req.ConditionList))
		for i, condition := range req.ConditionList {
			conditionList[i] = custom_types.Condition{
				Equal:       condition.Equal,
				LabelIdList: condition.LabelIdList,
			}
		}
	}

	var actionMessage []custom_types.Action
	if len(req.ActionMessage) > 0 {
		actionMessage = make([]custom_types.Action, len(req.ActionMessage))
		for i, condition := range req.ActionMessage {
			actionMessage[i] = custom_types.Action{
				Type:    condition.Type,
				Content: condition.Content,
			}
		}
	}

	newStage, err := l.svcCtx.DB.SopStage.Create().
		SetNotNilStatus(req.Status).
		SetNotNilTaskID(req.TaskId).
		SetNotNilName(req.Name).
		SetNotNilConditionType(req.ConditionType).
		SetNotNilConditionOperator(req.ConditionOperator).
		SetNotNilConditionList(conditionList).
		SetNotNilActionMessage(actionMessage).
		SetNotNilActionLabel(req.ActionLabel).
		SetNotNilIndexSort(&count).
		Save(l.ctx)

	if err != nil {
		return nil, dberrorhandler.DefaultEntError(l.Logger, err, req)
	}

	return &types.SopStageCreateResp{BaseDataInfo: types.BaseDataInfo{Msg: errormsg.CreateSuccess}, Data: newStage.ID}, nil
}
