package sop_stage

import (
	"context"
	"github.com/suyuan32/simple-admin-common/msg/errormsg"
	"github.com/suyuan32/simple-admin-common/utils/pointy"
	"wechat-api/ent/sopstage"
	"wechat-api/internal/utils/dberrorhandler"

	"wechat-api/internal/svc"
	"wechat-api/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetSopStageDetailLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewGetSopStageDetailLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetSopStageDetailLogic {
	return &GetSopStageDetailLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx}
}

func (l *GetSopStageDetailLogic) GetSopStageDetail(req *types.IDReq) (resp *types.SopStageInfoResp, err error) {
	data, err := l.svcCtx.DB.SopStage.Query().
		Where(sopstage.ID(req.Id)).
		WithStageNodes().
		Only(l.ctx)
	if err != nil {
		return nil, dberrorhandler.DefaultEntError(l.Logger, err, req)
	}

	var conditionList []types.Condition
	if len(data.ConditionList) > 0 {
		conditionList = make([]types.Condition, len(data.ConditionList))
		for i, condition := range data.ConditionList {
			conditionList[i] = types.Condition{
				Equal:       condition.Equal,
				LabelIdList: condition.LabelIdList,
			}
		}
	}

	var actionMessage []types.Action
	if len(data.ActionMessage) > 0 {
		actionMessage = make([]types.Action, len(data.ActionMessage))
		for i, condition := range data.ActionMessage {
			actionMessage[i] = types.Action{
				Type:    condition.Type,
				Content: condition.Content,
			}
		}
	}

	return &types.SopStageInfoResp{
		BaseDataInfo: types.BaseDataInfo{
			Code: 0,
			Msg:  errormsg.Success,
		},
		Data: types.SopStageInfo{
			BaseIDInfo: types.BaseIDInfo{
				Id:        &data.ID,
				CreatedAt: pointy.GetPointer(data.CreatedAt.UnixMilli()),
				UpdatedAt: pointy.GetPointer(data.UpdatedAt.UnixMilli()),
			},
			Status:            &data.Status,
			TaskId:            &data.TaskID,
			Name:              &data.Name,
			ConditionType:     &data.ConditionType,
			ConditionOperator: &data.ConditionOperator,
			ConditionList:     conditionList,
			ActionMessage:     actionMessage,
			ActionLabel:       data.ActionLabel,
			IndexSort:         &data.IndexSort,
		},
	}, nil
}
