package sop_stage

import (
	"context"
	"wechat-api/ent/custom_types"
	"wechat-api/ent/sopstage"

	"wechat-api/internal/svc"
	"wechat-api/internal/types"
	"wechat-api/internal/utils/dberrorhandler"

	"github.com/suyuan32/simple-admin-common/msg/errormsg"
	"github.com/zeromicro/go-zero/core/logx"
)

type UpdateSopStageLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewUpdateSopStageLogic(ctx context.Context, svcCtx *svc.ServiceContext) *UpdateSopStageLogic {
	return &UpdateSopStageLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *UpdateSopStageLogic) UpdateSopStage(req *types.SopStageInfo) (*types.BaseMsgResp, error) {
	// 根据 id 关联查询 task 信息，并判断 task 的状态是否为 1
	data, err := l.svcCtx.DB.SopStage.Query().
		Where(sopstage.ID(*req.Id)).
		WithSopTask().
		Only(l.ctx)
	if err != nil {
		return nil, dberrorhandler.DefaultEntError(l.Logger, err, req)
	}
	task := data.Edges.SopTask
	if task.Status != 1 {
		return nil, dberrorhandler.DefaultEntError(l.Logger, err, req)
	}

	var conditionList []custom_types.Condition
	if len(req.ConditionList) > 0 {
		conditionList = make([]custom_types.Condition, len(req.ConditionList))
		for i, condition := range req.ConditionList {
			conditionList[i] = custom_types.Condition{
				Equal:       condition.Equal,
				LabelIdList: condition.LabelIdList,
			}
		}
	}

	var actionMessage []custom_types.Action
	if len(req.ActionMessage) > 0 {
		actionMessage = make([]custom_types.Action, len(req.ActionMessage))
		for i, condition := range req.ActionMessage {
			actionMessage[i] = custom_types.Action{
				Type:    condition.Type,
				Content: condition.Content,
			}
		}
	}

	err = l.svcCtx.DB.SopStage.UpdateOneID(*req.Id).
		SetNotNilStatus(req.Status).
		SetNotNilTaskID(req.TaskId).
		SetNotNilName(req.Name).
		SetNotNilConditionType(req.ConditionType).
		SetNotNilConditionOperator(req.ConditionOperator).
		SetNotNilConditionList(conditionList).
		SetNotNilActionMessage(actionMessage).
		SetNotNilActionLabel(req.ActionLabel).
		SetNotNilIndexSort(req.IndexSort).
		Exec(l.ctx)

	if err != nil {
		return nil, dberrorhandler.DefaultEntError(l.Logger, err, req)
	}

	return &types.BaseMsgResp{Msg: errormsg.UpdateSuccess}, nil
}
