package sop_task

import (
	"context"
	"fmt"
	"wechat-api/internal/svc"
	"wechat-api/internal/types"
	"wechat-api/internal/utils/dberrorhandler"

	"github.com/suyuan32/simple-admin-common/msg/errormsg"
	"github.com/suyuan32/simple-admin-common/utils/pointy"
	"github.com/zeromicro/go-zero/core/logx"
)

type UpdateSopTaskLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewUpdateSopTaskLogic(ctx context.Context, svcCtx *svc.ServiceContext) *UpdateSopTaskLogic {
	return &UpdateSopTaskLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *UpdateSopTaskLogic) UpdateSopTask(req *types.SopTaskInfo) (*types.BaseMsgResp, error) {
	// 判断 planEndTime 和 planStartTime 是否非空，且 planEndTime 是否大于 planStartTime
	if req.PlanEndTime != nil && req.PlanStartTime != nil && (*req.PlanEndTime <= *req.PlanStartTime) {
		return nil, dberrorhandler.DefaultEntError(l.Logger, fmt.Errorf("planEndTime must be greater than planStartTime"), req)
	}

	// 根据 id 查询 task 信息，如果 Status 的值不为 1，则不允许修改
	task, err := l.svcCtx.DB.SopTask.Get(l.ctx, *req.Id)
	if err != nil {
		return nil, dberrorhandler.DefaultEntError(l.Logger, err, req)
	}
	if task.Status != 1 {
		return nil, dberrorhandler.DefaultEntError(l.Logger, err, req)
	}

	err = l.svcCtx.DB.SopTask.UpdateOneID(*req.Id).
		SetNotNilStatus(req.Status).
		SetNotNilName(req.Name).
		SetNotNilBotWxidList(req.BotWxidList).
		SetNotNilType(req.Type).
		SetNotNilPlanStartTime(pointy.GetTimeMilliPointer(req.PlanStartTime)).
		SetNotNilPlanEndTime(pointy.GetTimeMilliPointer(req.PlanEndTime)).
		SetNotNilCreatorID(req.CreatorId).
		Exec(l.ctx)

	if err != nil {
		return nil, dberrorhandler.DefaultEntError(l.Logger, err, req)
	}

	return &types.BaseMsgResp{Msg: errormsg.UpdateSuccess}, nil
}
